from subprocess import Popen, PIPE
import os

from django.conf import settings
from django.utils.importlib import import_module

from djangomaster.conf import settings as master_settings
from djangomaster.views import MasterView


class Result:

    def __init__(self, file_path, cmd=master_settings.PYLINT_CMD):
        self._path = file_path
        self._cmd = cmd
        self._result = ''

    @property
    def filename(self):
        index = len(master_settings.BASE_DIR) + 1
        return self._path[index:]

    @property
    def result(self):
        if not self._result:
            self._result = self.execute()
        return self._result

    def execute(self):
        cmd = [self._cmd, self._path]
        return Popen(cmd, stdout=PIPE).stdout.read()


class BaseLintView(MasterView):
    template_name = 'djangomaster/lint.html'
    context_object_name = 'results'
    menu_item = 'lint'
    lint_cmd = None
    lint_is_installed = False
    title = ''

    def is_valid_file(self, file_path):
        return True

    def lint(self, file_path):
        return Result(file_path, cmd=self.lint_cmd)

    def get_app_dirs(self):
        dirs = []
        for app in settings.INSTALLED_APPS:
            try:
                mod = import_module(app)
                dirs.append(mod.__path__[0])
            except ImportError:
                pass # What to do ?

        return dirs

    def get_files(self, dirname):
        ret = []

        for root, dirs, files in os.walk(dirname):
            for filename in files:
                file_path = os.path.join(root, filename)
                if self.is_valid_file(file_path):
                    ret.append(self.lint(file_path))

        return ret

    def get_queryset(self):
        ret = []
        if not master_settings.BASE_DIR:
            return ret

        dirs = sorted(self.get_app_dirs())
        for dirname in dirs:
            path = os.path.join(master_settings.BASE_DIR, dirname)
            if not dirname.startswith(master_settings.BASE_DIR):
                continue # to execute in only local code

            ret.extend(self.get_files(dirname))

        return ret

    def get_context_data(self, **kwargs):
        context = super(BaseLintView, self).get_context_data(**kwargs)
        context['page_title'] = self.title
        context['LINT_IS_INSTALLED'] = self.lint_is_installed
        context['LINT_CMD'] = self.lint_cmd
        return context


class PyLintView(BaseLintView):
    menu_item = 'pylint'
    lint_cmd = master_settings.PYLINT_CMD
    lint_is_installed = master_settings.PYLINT_IS_INSTALLED
    title = ''
    title = 'Py Lint'

    def is_valid_file(self, file_path):
        return file_path.endswith('.py')


class JsLintView(BaseLintView):
    menu_item = 'jslint'
    lint_cmd = master_settings.JSLINT_CMD
    lint_is_installed = master_settings.JSLINT_IS_INSTALLED
    title = 'JS Lint'

    def is_valid_file(self, file_path):
        return file_path.endswith('.js')




from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.generic.detail import BaseDetailView
from django.views.generic.list import ListView

from djangomaster.conf import settings as conf


class MasterView(ListView):

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied

        return super(MasterView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(MasterView, self).get_context_data(**kwargs)
        context['menu_item'] = getattr(self, 'menu_item', '')
        context['settings'] = conf
        return context


class JSONResponseMixin(object):
    """
    # @see https://docs.djangoproject.com/en/1.4/topics/class-based-views/
    """

    def render_to_response(self, context):
        "Returns a JSON response containing 'context' as payload"
        return self.get_json_response(self.convert_context_to_json(context))

    def get_json_response(self, content, **httpresponse_kwargs):
        "Construct an `HttpResponse` object."
        return HttpResponse(content, content_type='application/json',
                            **httpresponse_kwargs)

    def convert_context_to_json(self, context):
        "Convert the context dictionary into a JSON object"
        # Note: This is *EXTREMELY* naive; in reality, you'll need
        # to do much more complex handling to ensure that arbitrary
        # objects -- such as Django model instances or querysets
        # -- can be serialized as JSON.

        if 'object' in context:
            return json.dumps(context['object'], cls=DjangoJSONEncoder)
        else:
            return json.dumps(context, cls=DjangoJSONEncoder)


class JSONDetailView(JSONResponseMixin, BaseDetailView):
    pass


class MasterJSONView(JSONResponseMixin, ListView):
    @method_decorator(login_required)
    def dispach(self, *args, **kwargs):
        if not self.request.user.is_admin:
            raise PermissionDenied

        return super(JSONDetailView, self).dispach(*args, **kwargs)



import os
from md5 import md5
from subprocess import Popen, PIPE

from django.conf import settings as djangosettings
from django.core.exceptions import ImproperlyConfigured


TEST_STATIC_FOLDER = os.path.join(djangosettings.STATIC_ROOT, 'djangomaster',
                                  md5(djangosettings.SECRET_KEY).hexdigest())

TEST_STATIC_FOLDER = getattr(djangosettings, 'TEST_STATIC_FOLDER',
                             TEST_STATIC_FOLDER)


def _is_installed(cmd):
    try:
        resp = Popen(['which', cmd], stdout=PIPE)
    except OSError:
        return False
    return resp.stdout.read().strip().endswith(cmd)


def _create_whole_path_to_folder(path, start_from=''):
    """Creates the whole path until the given path is satisfied"""
    dirs = []
    while True:
        paths = os.path.split(path)
        path = paths[0]
        dirs.append(paths[1])
        if path == start_from or path == '':
            break

    path = start_from
    dirs.reverse()
    for name in dirs:
        path = os.path.join(path, name)
        if not os.path.isdir(path):
            os.mkdir(path)


class Settings():
    conf = djangosettings

    @property
    def BASE_DIR_NAME(self):
        return getattr(self.conf, 'DJANGOMASTER_BASE_DIR', 'BASE_DIR')

    @property
    def BASE_DIR(self):
        return getattr(self.conf, self.BASE_DIR_NAME, None)

    @property
    def REQUIREMENTS_TXT(self):
        return getattr(self.conf, 'DJANGOMASTER_REQUIREMENTS_NAME',
                       'requirements.txt')

    @property
    def PYLINT_CMD(self):
        return getattr(self.conf, 'DJANGOMASTER_PYLINT_CMD', 'pep8')

    @property
    def PYLINT_IS_INSTALLED(self):
        return _is_installed(self.PYLINT_CMD)

    @property
    def JSLINT_CMD(self):
        return getattr(self.conf, 'DJANGOMASTER_JSLINT_CMD', 'jshint')

    @property
    def JSLINT_IS_INSTALLED(self):
        return _is_installed(self.JSLINT_CMD)

    @property
    def SOUTH_IS_INSTALLED(self):
        return 'south' in self.conf.INSTALLED_APPS

    @property
    def SIGNAL_MODULES(self):
        modules = getattr(self.conf, 'DJANGOMASTER_SIGNAL_MODULES', ())
        if type(modules) not in (tuple, list):
            raise ImproperlyConfigured('DJANGOMASTER_SIGNAL_MODULES must be '
                                       'a tuple')
        return modules

    @property
    def TEST_STATIC_FOLDER(self):
        _create_whole_path_to_folder(start_from=djangosettings.STATIC_ROOT,
                                     path=TEST_STATIC_FOLDER)
        return TEST_STATIC_FOLDER


settings = Settings()



import os

from djangomaster.conf import settings
from djangomaster.views import MasterView


class HomeView(MasterView):
    template_name = 'djangomaster/home.html'
    context_object_name = 'object'
    menu_item = 'home'

    def get_queryset(self):
        ret = {}
        ret['requirements'] = self.get_requirements()

        return ret

    def get_requirements(self):
        if settings.BASE_DIR is None:
            return {'BASE_DIR_ERROR': True}

        path = os.path.join(settings.BASE_DIR, settings.REQUIREMENTS_TXT)
        if not os.path.isfile(path):
            return {'REQUIREMENTS_ERROR': True, 'path': path}

        return {'content': open(path).read()}



from django.conf.urls import patterns, url

from djangomaster.views.home import HomeView
from djangomaster.views.routes import RoutesListView
from djangomaster.views.settings import SettingsListView
from djangomaster.views.signals import SignalsListView
from djangomaster.views.lint import PyLintView, JsLintView
from djangomaster.views.test import TestView, ExecuteTestView, CheckTestView
from djangomaster.views.south import SouthView


urlpatterns = patterns(
    r'',
    url(r'^$', HomeView.as_view(), name='home'),
    url(r'^routes/$', RoutesListView.as_view(), name='routes'),
    url(r'^settings/$', SettingsListView.as_view(), name='settings'),
    url(r'^pylint/$', PyLintView.as_view(), name='pylint'),
    url(r'^jslint/$', JsLintView.as_view(), name='jslint'),
    url(r'^signals/$', SignalsListView.as_view(), name='signals'),
    url(r'^south/$', SouthView.as_view(), name='south'),

    # Tests
    url(r'^test/$', TestView.as_view(), name='test'),
    url(r'^test/execute/$', ExecuteTestView.as_view(), name='test_execute'),
    url(r'^test/check/$', CheckTestView.as_view(), name='test_check'),
)



import os
from subprocess import Popen
import tempfile

from djangomaster.views import MasterView, MasterJSONView
from djangomaster.conf import settings


class TestView(MasterView):
    template_name = 'djangomaster/test.html'
    menu_item = 'test'

    def get_context_data(self, **kwargs):
        context = super(TestView, self).get_context_data(**kwargs)
        context['cmd'] = self.request.GET.get('cmd', 'test')

        return context

    def get_queryset(self):
        return []


class ExecuteTestView(MasterJSONView):

    def get_queryset(self):
        return []

    def get_context_data(self, **kwargs):
        context = super(ExecuteTestView, self).get_context_data(**kwargs)
        context['cmd'] = self.request.GET.get('cmd', 'test')
        file_path = self._execute(context['cmd'])
        context['file'] = os.path.split(file_path)[1]
        return context

    def get_random_file(self):
        _file = tempfile.NamedTemporaryFile(prefix='test', suffix='.txt',
                                            dir=settings.TEST_STATIC_FOLDER,
                                            delete=False)
        return open(_file.name, 'wr')

    def _execute(self, cmd):
        # check this to improvement:
        # http://stackoverflow.com/questions/2581817/python-subprocess-callback-when-cmd-exits
        cmd = ['python', 'manage.py'] + cmd.split(' ')
        f = self.get_random_file()
        Popen(cmd, stdout=f, stderr=f)
        return f.name


class CheckTestView(MasterJSONView):

    def get_queryset(self):
        return []

    def get_context_data(self, **kwargs):
        ctx = super(CheckTestView, self).get_context_data(**kwargs)
        file_path = self.request.GET.get('file', '')
        file_path = os.path.join(settings.TEST_STATIC_FOLDER, file_path)

        try:
            with open(file_path, 'r') as f:
                ctx['output'] = f.read()
                ctx['finished'] = 'Destroying test database' in ctx['output']
        except IOError:
            ctx['output'] = 'File doesnt exist'
            ctx['finished'] = True

        return ctx



from djangomaster.views import MasterView


class SouthView(MasterView):
    template_name = 'djangomaster/south.html'

    def get_context_data(self, **kwargs):
        context = super(SouthView, self).get_context_data(**kwargs)
        context['history_list'] = history_list

        return context

    def get_queryset(self):
        return []


def history_list():
    """
    Based in:
    https://github.com/dmishe/django-south/blob/master/south/management/commands/migrate.py#L108
    """
    from south import migration
    from south.models import MigrationHistory

    ret = {}

    apps = migration.all_migrations()
    labels = [app.app_label() for app in migration.all_migrations()]

    applied_migrations = []
    for mi in MigrationHistory.objects.filter(app_name__in=labels):
        applied_migrations.append('%s.%s' % (mi.app_name, mi.migration))

    for app in apps:
        label = app.app_label()
        ret[label] = []

        for mi in app:
            migration_name = mi.app_label() + "." + mi.name()
            applied = migration_name in applied_migrations
            ret[label].append({'name': migration.name(), 'applied': applied})

    return ret



from django.dispatch import Signal

from djangomaster.conf import settings
from djangomaster.views import MasterView

SIGNAL_MODULES = (
    'django.core.signals',
    'django.contrib.auth.signals',
    'django.test.signals',
    'django.db.models.signals',
    'django.db.backends.signals',
    'django.contrib.comments.signals',
) + settings.SIGNAL_MODULES


def is_signal(instance):
    """Given the instance, checks if it is or any
    base class is the Signal class """
    cls = getattr(instance, '__class__', instance)

    if cls in (type, object):
        return False

    if cls == Signal:
        return True

    for base in cls.__bases__:
        if is_signal(base):
            return True

    return False


def get_module(module_name):
    paths = module_name.split('.')
    module = __import__(module_name)
    for path in paths[1:]:
        module = getattr(module, path)
    return module


class SignalsListView(MasterView):
    template_name = 'djangomaster/signals.html'
    context_object_name = 'signal_list'
    menu_item = 'signals'

    def get_queryset(self):
        ret = []
        for module_name in SIGNAL_MODULES:
            module = get_module(module_name)
            for name, item in module.__dict__.items():
                if is_signal(item):
                    ret.append(SignalInstance(item, name=name,
                                              module=module_name))
        return ret


class SignalInstance:

    def __init__(self, instance, name='', module=''):
        self.instance = instance
        self._name = name
        self._module = module

    @property
    def name(self):
        cls_name = self._module
        if self._name:
            cls_name += '.' + self._name
        return cls_name

    @property
    def receivers(self):
        ret = []
        for receiver in self.instance.receivers:
            ret.append(Receiver(receiver))
        return ret

    def has_receivers(self):
        return len(self.instance.receivers) > 0


class Receiver:

    def __init__(self, receiver):
        self.receiver = receiver
        self._func = receiver[1]()

    @property
    def name(self):
        return self._func.func_name

    @property
    def file_name(self):
        return "%s:%s" % (self._func.func_code.co_filename,
                          self._func.func_code.co_firstlineno)



import djangomaster

from django.conf import settings
from django.conf.urls import patterns, url, include
from django.conf.urls.static import static
from django.contrib import admin


admin.autodiscover()


urlpatterns = patterns(
    r'',

    url(r'^admin/', include(admin.site.urls)),
    url(r'^master/', include(djangomaster.urls)),

)


if settings.DEBUG:
    urlpatterns += static(settings.STATIC_URL,
                          document_root=settings.STATIC_ROOT)



try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

from django.conf import settings

from djangomaster.views import MasterView


class SettingsListView(MasterView):
    template_name = 'djangomaster/settings.html'
    context_object_name = 'settings_list'
    menu_item = 'settings'

    def get_queryset(self):
        ret = OrderedDict()
        items = settings._wrapped.__dict__.items()

        for key, value in sorted(items, key=lambda key: key):
            ret[key] = value

        return ret.items()
